#ifndef PANZER_CORE_CONFIG_HPP
#define PANZER_CORE_CONFIG_HPP

#ifndef __cplusplus
#define __cplusplus
#endif

#define HAVE_MPI
#define PANZER_HAVE_EPETRA

// What AD Type are you going to use?
#define PANZER_FADTYPE Sacado::Fad::DFad<RealType>
/* #undef PANZER_DEBUG */
#define HAVE_PANZER_EXPLICIT_INSTANTIATION
/* #undef PANZER_EXPLICIT_INSTANTIATION */

#include "Tpetra_Details_DefaultTypes.hpp" // for ordinal types

namespace panzer {

  // Tpetra deprecated code prefers "int" over "long long" for global
  // ordinal, while non-deprecated tpetra and panzer prefer "long
  // long" over "int". This code will choose long long over int. When
  // tpetra removes deprecated code, we can just default directly to
  // tpetra.
#if defined(HAVE_TPETRA_INST_INT_LONG_LONG)
  using GlobalOrdinal = long long int;
#else
  using GlobalOrdinal = Tpetra::Details::DefaultTypes::global_ordinal_type;
#endif

  using LocalOrdinal = int;
  using index_t = int;
}

#ifndef PANZER_FUNC_TIME_MONITOR
#  define PANZER_TEUCHOS_TIME_MONITOR
#  define PANZER_FUNC_TIME_MONITOR(FUNCNAME) \
     TEUCHOS_FUNC_TIME_MONITOR_DIFF(FUNCNAME, PANZER)
#  define PANZER_FUNC_TIME_MONITOR_DIFF(FUNCNAME, DIFF) \
     TEUCHOS_FUNC_TIME_MONITOR_DIFF(FUNCNAME, DIFF)
#endif

#ifndef PANZER_DEPRECATED
#  if (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 1))
#    define PANZER_DEPRECATED  __attribute__((__deprecated__))
#  else
#    define PANZER_DEPRECATED
#  endif
#endif

#ifndef PANZER_DEPRECATED_MSG
#  if (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 5))
#    define PANZER_DEPRECATED_MSG(MSG)  __attribute__((__deprecated__ (#MSG) ))
#  elif (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 1))
#    define PANZER_DEPRECATED_MSG(MSG)  __attribute__((__deprecated__))
#  else
#    define PANZER_DEPRECATED_MSG(MSG)
#  endif
#endif


#endif
