# @HEADER
# ************************************************************************
#
#            TriBITS: Tribal Build, Integrate, and Test System
#                    Copyright 2013 Sandia Corporation
#
# Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
# the U.S. Government retains certain rights in this software.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
# 1. Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the Corporation nor the names of the
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# ************************************************************************
# @HEADER

##############################################################################
#
# CMake variable for use by Trilinos/Triutils clients.
#
# Do not edit: This file was generated automatically by CMake.
#
##############################################################################

if(CMAKE_VERSION VERSION_LESS 3.3)
  set(${CMAKE_FIND_PACKAGE_NAME}_NOT_FOUND_MESSAGE
    "Triutils requires CMake 3.3 or later for 'if (... IN_LIST ...)'"
    )
  set(${CMAKE_FIND_PACKAGE_NAME}_FOUND FALSE)
  return()
endif()
cmake_minimum_required(VERSION 3.3...3.17.0)

## ---------------------------------------------------------------------------
## Compilers used by Trilinos/Triutils build
## ---------------------------------------------------------------------------

set(Triutils_CXX_COMPILER "/usr/lib64/mpich/bin/mpicxx")

set(Triutils_C_COMPILER "/usr/lib64/mpich/bin/mpicc")

set(Triutils_Fortran_COMPILER "/usr/lib64/mpich/bin/mpif90")
# Deprecated!
set(Triutils_FORTRAN_COMPILER "/usr/lib64/mpich/bin/mpif90") 


## ---------------------------------------------------------------------------
## Compiler flags used by Trilinos/Triutils build
## ---------------------------------------------------------------------------

## Give the build type
set(Triutils_CMAKE_BUILD_TYPE "Release")

## Set compiler flags, including those determined by build type
set(Triutils_CXX_FLAGS [[ ]])

set(Triutils_C_FLAGS [[ -pedantic -Wall -Wno-long-long -std=c99   -O2 -flto=auto -ffat-lto-objects -fexceptions  -grecord-gcc-switches -pipe -Wall  -Wp,-D_FORTIFY_SOURCE=2 -Wp,-D_GLIBCXX_ASSERTIONS    -m64 -mcpu=power8 -mtune=power8 -fasynchronous-unwind-tables -fstack-clash-protection -fPIC -w -I/usr/include/mpich-ppc64le -fopenmp ]])

set(Triutils_Fortran_FLAGS [[ ]])
# Deprecated
set(Triutils_FORTRAN_FLAGS [[ ]])

## Extra link flags (e.g., specification of fortran libraries)
set(Triutils_EXTRA_LD_FLAGS [[]])

## This is the command-line entry used for setting rpaths. In a build
## with static libraries it will be empty.
set(Triutils_SHARED_LIB_RPATH_COMMAND "-Wl,-rpath,/usr//usr/lib64/mpich/lib")
set(Triutils_BUILD_SHARED_LIBS "ON")

set(Triutils_LINKER /usr/bin/ld)
set(Triutils_AR /usr/bin/ar)

## ---------------------------------------------------------------------------
## Set library specifications and paths
## ---------------------------------------------------------------------------

## Base install location (if not in the build tree)
set(Triutils_INSTALL_DIR "/usr")

## List of package include dirs
set(Triutils_INCLUDE_DIRS "")

## List of package library paths
set(Triutils_LIBRARY_DIRS "")

## List of package libraries
set(Triutils_LIBRARIES "Triutils::triutils;Epetra::epetra;TeuchosKokkosComm::teuchoskokkoscomm;TeuchosKokkosCompat::teuchoskokkoscompat;TeuchosRemainder::teuchosremainder;TeuchosNumerics::teuchosnumerics;TeuchosComm::teuchoscomm;TeuchosParameterList::teuchosparameterlist;TeuchosParser::teuchosparser;TeuchosCore::teuchoscore;KokkosCore::kokkoscore")

## Specification of directories for TPL headers
set(Triutils_TPL_INCLUDE_DIRS "")

## Specification of directories for TPL libraries
set(Triutils_TPL_LIBRARY_DIRS "")

## List of required TPLs
set(Triutils_TPL_LIBRARIES "DLlib::all_libs;Eigen::all_libs;Boost::all_libs;LAPACK::all_libs;BLAS::all_libs;MPI::all_libs;Pthread::all_libs;yaml-cpp::all_libs")

## ---------------------------------------------------------------------------
## MPI specific variables
##   These variables are provided to make it easier to get the mpi libraries
##   and includes on systems that do not use the mpi wrappers for compiling
## ---------------------------------------------------------------------------

set(Triutils_MPI_LIBRARIES "")
set(Triutils_MPI_LIBRARY_DIRS "")
set(Triutils_MPI_INCLUDE_DIRS "")
set(Triutils_MPI_EXEC "/usr/lib64/mpich/bin/mpiexec")
set(Triutils_MPI_EXEC_MAX_NUMPROCS "4")
set(Triutils_MPI_EXEC_NUMPROCS_FLAG "-np")

## ---------------------------------------------------------------------------
## Set useful general variables
## ---------------------------------------------------------------------------

## The packages enabled for this project
set(Triutils_PACKAGE_LIST "Triutils;Epetra;Teuchos;TeuchosKokkosComm;TeuchosKokkosCompat;TeuchosRemainder;TeuchosNumerics;TeuchosComm;TeuchosParameterList;TeuchosParser;TeuchosCore;KokkosCore")

## The TPLs enabled for this project
set(Triutils_TPL_LIST "DLlib;Eigen;Boost;LAPACK;BLAS;MPI;Pthread;yaml-cpp")

# Enables/Disables for upstream package dependencies
set(Triutils_ENABLE_TeuchosCore ON)
set(Triutils_ENABLE_Epetra ON)

# Exported cache variables

# Include configuration of dependent packages
if (NOT TARGET TeuchosCore::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../TeuchosCore/TeuchosCoreConfig.cmake")
endif()
if (NOT TARGET Epetra::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Epetra/EpetraConfig.cmake")
endif()

# Import Triutils targets
include("${CMAKE_CURRENT_LIST_DIR}/TriutilsTargets.cmake")

## ----------------------------------------------------------------------------
## Create deprecated non-namespaced library targets for backwards compatibility
## ----------------------------------------------------------------------------

set(Triutils_EXPORTED_PACKAGE_LIBS_NAMES "triutils")

foreach(libname IN LISTS Triutils_EXPORTED_PACKAGE_LIBS_NAMES)
  if (NOT TARGET ${libname})
    add_library(${libname} INTERFACE IMPORTED)
    target_link_libraries(${libname}
       INTERFACE Triutils::${libname})
    set(deprecationMessage
      "WARNING: The non-namespaced target '${libname}' is deprecated!"
      "  If always using newer versions of the project 'Trilinos', then use the"
      " new namespaced target 'Triutils::${libname}', or better yet,"
      " 'Triutils::all_libs' to be less sensitive to changes in the definition"
      " of targets in the package 'Triutils'.  Or, to maintain compatibility with"
      " older or newer versions the project 'Trilinos', instead link against the"
      " libraries specified by the variable 'Triutils_LIBRARIES'."
      )
    string(REPLACE ";" "" deprecationMessage "${deprecationMessage}")
    set_target_properties(${libname}
      PROPERTIES DEPRECATION "${deprecationMessage}" )
  endif()
endforeach()
